#ifndef SCANNERSC2HDM_H
#define SCANNERSC2HDM_H
#include <iostream>
#include "../../ScannerSInterface.h"
#include "../../ScannerSModelclasses.h"
#include "../../ScannerSTables.h"
#include "../../ScannerSEWPO.h"
#include "../../ScannerSAnalysisUtils.h"
#include "../../ScannerSAux.h"
#include "../../ScannerS_exp_values.h"
#include <gsl/gsl_multimin.h>
#include <gsl/gsl_multiroots.h>
#include <algorithm>
#include <array>
#include <vector>
#include <map>




#include "../../ExtInterfaces/Wrappers.h"


class C2HDM:public Potential{
//UNDER CONSTRUCTION!!!!!!!!!!!!
    static std::string filepath;
    static const unsigned int nHzero = 3;      //Number of neutral scalars
    static const unsigned int nHplus=1;		    //Number of singly charged scalars
    static const int whichanalyses=3;	//which Higgs bounds analysis
    static const int whichinput=1; 	  //which type of HB input you will provide (1=hadronic)
    static const unsigned int nDoublet = 2; // Number of doublets, needed for STUchecker
    static const double Ecm2;
    static const double Ecm7;
    static const double Ecm8;




    std::array<double, nHzero> Rhvv;
    std::array<std::array<double,nHzero>, nHzero> Rhhv;
	//Higgs couplings to fermions indices are:
  // Rhf[iHiggs][ifermion][CP]: iHiggs=0,1,2 (A,H1,H2), ifermion= 0,1,2 (up-type quarks, d-type quarks and leptons), CP = 1, -1
    std::array<std::array<std::array<double,2> ,3>, nHzero> Rhf;

    std::map<std::string,double> mu_results;
    STUchecker STU_C2HDM{};

    double bsgamma;
#ifdef HBMODE
    HBobject HiggsBoundsAnalysis;
    void PrepareHiggsBoundsInput();
    int HB_channel;
#endif
    std::array<bool,3> pseudo_bounds;//initialised in readfile
    std::array<double,3> pseudo_min;
    std::array<double,3> pseudo_max;
    std::array<bool,5> mass_bounds;//initialised in readfile
    std::array<bool,10> L_bounds;//initialised in readfile
    std::array<bool,8> phi_bounds {{1,1,0,1,1,1,0,1}};
    int Type2HDM;
    std::array<std::array<double,3>, 3> Rmix;

    double mHc,beta,tanbeta;
    //mass ordering
    double mH1,mH2,mH3,a1,a2,a3;

    //tree uni maxvalue
    double maxev;
    //electron EDM
    double eEDM;

    //whether m2 reparametrisation is active
    bool m1_reparam;

    //counters to show which constraint kills how many points
    static int fail_global,fail_STU,fail_Z_bbar,fail_bs_gamma,fail_HB,fail_rates,fail_EDM;


  public:
    std::map<std::string,double> Hdecay;

    C2HDM();
    void PrintHeader()const;
    void readfile(std::istream & in);
    void UserInitCalcs();
    void UserFinalCalcs();
    void what2print();
    bool CheckBounds()const;
    bool CheckStability(LambdaRef & L)const;
    bool CheckGlobal(PhiRef & Phi,LambdaRef & L,Potential & V)const;
    bool UserAnalysis(PhiRef & Phi,LambdaRef & L,MassRef & Mass, MmixingRef & Mixing);
    std::string name() const;
    bool CheckTreeUni();
    void MyPhiParametrization(const PhiParamVec & PhiPar,PhiVec & Phi);
    void MyInternalMixing(const PhiParamVec & PhiPar,const PhiVec & Phi,MixingparamVec & MixPar,std::vector< std::vector<double> > & MixInternal);
    void MyCoupMassRelations(const PhiParamVec & PhiPar,const PhiVec & Phi,const std::vector< std::vector<double> > & Mixing,const MixingparamVec & MixPar,std::vector<double> & L,std::vector<double> & Mass, MLparamVec & MLPar, std::vector<double> & IndependentMs,std::vector<double> & IndependentLs);

  private:
    void PhysicalStatesC2HDM();
    bool TestZbbarbLEP() const;
    bool Test_bs_gamma();
    bool CheckSignalStrengths();
    bool TestSTU();

    bool TestEDM();
    double fermion_loops();
    double Hc_loops();
    double W_loops();
    double HW_loops();
    double func_I(double m1sq,double m2sq,int i);
};


// SuperisoTables for B->X_s gamma to be available for User

extern TableData TableSuperisoBsgammaType1;
extern TableData TableSuperisoBsgammaType2;
extern TableData TableSuperisoBsgammaType3;
extern TableData TableSuperisoBsgammaType4;

// functions for EDM
double integrand_I1(double x, void * params);
double integrand_I2(double x, void * params);
double integrand_I4(double x, void * params);
double integrand_I5(double x, void * params);
struct int_params{double m1sq;double m2sq;double mHcsq; double mWsq;};


#endif
