#include "ScannerSxSM2.h"

using namespace std;

string xSM2::filepath="ScannerScore/ScannerSModels/xSM2/xSM2.potential";
int xSM2::nHzero;
int xSM2::nHplus=0;
int xSM2::whichanalyses=3;	//which Higgs bounds analysis
int xSM2::whichinput=1; 	//which type of input you will provide (1=hadronic)

// CONSTRUCTOR
xSM2::xSM2():Gamma_const(1./32./acos(-1)){

  //heavy particle masses
  for(int i=0;i!=3;++i)
    mass_bounds[i]=false;
  //goldstone masses
  for(int i=3;i!=6;++i)
    mass_bounds[i]=true;
  //couplings
  for(int i=0;i!=7;++i)
    L_bounds[i]=false;
  //VeV's: 0 to 3 are Higgs' (0,1 and 3 being fixed at 0) and 4:5 are the new Singlet's
  phi_bounds[0]=true;
  phi_bounds[1]=true;
  phi_bounds[2]=false;
  phi_bounds[3]=true;
  phi_bounds[4]=false;
  phi_bounds[5]=false;
}

std::string xSM2::name() const{
//returns the name of the type of potential used.  
  string name("xSM2");	
  return name;
}

void xSM2::readfile(std::istream & in){
//This function is called from the overloaded operator >> when a xSM2 input file is provided.
//On the base class, Potential::readfile simply calls openinput() on "in".
//Since TwoHDM files only contain the relevante variables, "this" readfile opens an auxiliary file in the same format has Potential::readfile
//and then replaces the relevante variables bty the values in "in".
  ifstream modelfile;
  OpenModelFile(modelfile,filepath);
  openinput(modelfile);
  
  //these variables are independent and therefore their ranges are always needed
  phi_bounds[2]=getvariable(in,Phi.Vecmin[2],Phi.Vecmax[2],">vh");
  phi_bounds[4]=getvariable(in,Phi.Vecmin[4],Phi.Vecmax[4],">vs");
  phi_bounds[5]=getvariable(in,Phi.Vecmin[5],Phi.Vecmax[5],">va");
  //if we have a dark matter candidate, only 2 fields that couple directly with SM particles
  //Since mixes particles in crescent orther, in the DM case we need to have mHiggs at Mass[0] or Mass[1]
  if(Phi.Vecmin[5]==0 && Phi.Vecmax[5]==0){
    DM_candidate=true;
    nHzero=2;
    mass_bounds[0]=getvariable(in,Cp.Mass.Vecmin[0],Cp.Mass.Vecmax[0],">ms1");
    mass_bounds[1]=getvariable(in,Cp.Mass.Vecmin[1],Cp.Mass.Vecmax[1],">mh");
    mass_bounds[2]=getvariable(in,Cp.Mass.Vecmin[2],Cp.Mass.Vecmax[2],">ms2");    
  }
  //In the Mixall case we set mHiggs to Mass[2] since this is the only independent mass
  else{
    DM_candidate=false;
    nHzero=3;
    mass_bounds[0]=getvariable(in,Cp.Mass.Vecmin[0],Cp.Mass.Vecmax[0],">ms1");
    mass_bounds[1]=getvariable(in,Cp.Mass.Vecmin[1],Cp.Mass.Vecmax[1],">ms2");
    mass_bounds[2]=getvariable(in,Cp.Mass.Vecmin[2],Cp.Mass.Vecmax[2],">mh");
  }
  //these parameters will not be generated and will only be checked checkbounds (if they exist in the input file)
  L_bounds[0]=getvariable(in,Cp.L.Vecmin[0],Cp.L.Vecmax[0],">msq");
  L_bounds[1]=getvariable(in,Cp.L.Vecmin[1],Cp.L.Vecmax[1],">lambda");
  L_bounds[2]=getvariable(in,Cp.L.Vecmin[2],Cp.L.Vecmax[2],">delta2");
  L_bounds[3]=getvariable(in,Cp.L.Vecmin[3],Cp.L.Vecmax[3],">b2");
  L_bounds[4]=getvariable(in,Cp.L.Vecmin[4],Cp.L.Vecmax[4],">d2");
  L_bounds[5]=getvariable(in,Cp.L.Vecmin[5],Cp.L.Vecmax[5],">b1");  
  //a1 is only independent in the DM case
  L_bounds[6]=getvariable(in,Cp.L.Vecmin[6],Cp.L.Vecmax[6],">a1");
   
  if((!phi_bounds[2]) || (!phi_bounds[4]) || (!phi_bounds[5]) || (!mass_bounds[2])){
    cerr<<"ScannerS ERROR: Unable to find a necessary parameter in the xSM2 input file. This model ALWAYS requires you define the range for vh, vs, va and mh (aditionally you need to provide ranges for ms1, ms2 and a1 in the DM case). These are independent parameters that will be randomly generated."<<endl;
    exit(EXIT_FAILURE);
  }
  if(DM_candidate && ((!mass_bounds[0]) || (!mass_bounds[1]) || (!L_bounds[6]))){
    cerr<<"ScannerS ERROR: Unable to find a necessary parameter in the xSM2 input file. If you have a dark matter candidate (va=0), you need to provide ranges for ms1, ms2 and a1 in adition to the other needed parameters (vh, vs, va and mh). These are independent parameters that will be randomly generated."<<endl;
    exit(EXIT_FAILURE);
  }
#ifdef HBMODE
  HiggsBoundsAnalysis.rewrite_values(nHzero,nHplus,whichanalyses,whichinput); //sets the values needed to initialize HiggsBounds (will be initialized in UserInitCalcs)
#endif
}

void xSM2::PrintHeader() const{
  
  std::clog<<endl<<"***********************************************"<<endl;
  std::clog<<"*********** "<<scanners_version<<" ************"<<endl;
  std::clog<<"***********************************************" <<endl;
  std::clog<<endl<<"---------- SUMMARY INFO FOR THIS RUN ---------- "<<endl;
  std::clog<<endl<<"Potential Type: xSM2"<<endl;
  std::clog<<endl<<"V(H,S) = (L0/2)H'H + (L1/4)(H'H)² + (L2/2)H'H|S|² + (L3/2)|S|² + (L4/4)|S|⁴ + ((L5/4)S² + L6S + c.c.)"<<endl;
  std::clog<<endl<< "The expanded potential is:"<<endl;
  std::clog<<endl<<(*this)<<endl;
  std::clog<<endl<<"The ranges of parameters to be scanned are:"<<endl;
  std::clog<<endl<<"Ranges for vacuum expectation values:"<<endl;
  for(size_t i=0;i!=Phi.size();++i){
    if(phi_bounds[i])
      std::clog<<"phi"<<i<<" in ["<<Phi.vgetmin(i)<<":"<<Phi.vgetmax(i)<<"]"<< endl;
  }
  std::clog <<endl<<"Ranges for couplings:"<< endl;
  for(size_t i=0;i!=Cp.L.size();++i){
    if(L_bounds[i])
      std::clog<<"L"<<i<<" in ["<<Cp.L.vgetmin(i)<<":"<<Cp.L.vgetmax(i)<<"]"<< endl;
  }
  std::clog <<endl<<"Ranges for physical state masses:"<< endl;
  for(size_t i=0;i!=Cp.Mass.size();++i){
    if(mass_bounds[i])
    std::clog<<"m"<<i<<" in ["<<Cp.Mass.vgetmin(i)<<":"<<Cp.Mass.vgetmax(i)<<"]"<<endl;
  }
  std::clog<<endl;
  std::clog<<"The seed you are using for the random number generator is: "<<r.getseed()<< endl<< endl;

}

bool xSM2::CheckBounds() const{
//unlike Potential::CheckBounds(), this only checks dependent parameters for which the range was explicity declared in the xSM2 input file. if they were not declared in input file (and were loaded of xSM2.potential by default) mass_bounds and L_bounds will be false and the parameters will not be checked

  //in both cases (mix_all and DM_candidate) couplings 0:5 are dependent
  for(size_t i=0;i!=6;++i)
    if(L_bounds[i] && (Cp.L[i]<Cp.L.vgetmin(i) || Cp.L[i]>Cp.L.vgetmax(i))){
#ifdef VERBOSE
      std::clog<<endl<<"---> ScannerS: Point rejected by xSM2::CheckBounds()."<<endl;
#endif    
      return false;
    }
  
  //in the mix_all case coupling 6 and masses 0:1 are alse dependent (only mass 2 is independent)
  if(!DM_candidate){
    if(L_bounds[6] && (Cp.L[6]<Cp.L.vgetmin(6) || Cp.L[6]>Cp.L.vgetmax(6))){
#ifdef VERBOSE
      std::clog<<endl<<"---> ScannerS: Point rejected by xSM2::CheckBounds()."<<endl;
#endif
      return false;
    }
    if(mass_bounds[0] && (Cp.Mass[0]<Cp.Mass.vgetmin(0) || Cp.Mass[0]>Cp.Mass.vgetmax(0))){
#ifdef VERBOSE
      std::clog<<endl<<"---> ScannerS: Point rejected by xSM2::CheckBounds()."<<endl;
#endif
      return false;
    }
    if(mass_bounds[1] && (Cp.Mass[1]<Cp.Mass.vgetmin(1) || Cp.Mass[1]>Cp.Mass.vgetmax(1))){
#ifdef VERBOSE
      std::clog<<endl<<"---> ScannerS: Point rejected by xSM2::CheckBounds()."<<endl;
#endif      
      return false;
      }
  }
 
  return true;
}

void xSM2::beta_func(const double t, const std::vector<double> & y,std::vector<double> & dydt, void * par){
        
    double norm=1./(16.*acos(-1)*acos(-1));
    double bi[3]={41./10.,-19./6.,-7.};
    double ci[3]={-17./10.,-3./2.,-2};
    double bji[3][3]={{199./50.,9./10.,11./10.},{27./10.,35./6.,9./2.},{44./5.,12.,-26.}};
    
    //gammas
    double gammaH1= 3.*y[10]*y[10] - 9./20.*y[7]*y[7] - 9./4.*y[8]*y[8];
    double gammaH2= 1293./800.*y[7]*y[7]*y[7]*y[7] + 27./80.*y[7]*y[7]*y[8]*y[8] - 271./32.*y[8]*y[8]*y[8]*y[8] + 17./8.*y[7]*y[7]*y[10]*y[10] + 45./8.*y[8]*y[8]*y[10]*y[10] + 20.*y[9]*y[9]*y[10]*y[10] - 27./4.*y[10]*y[10]*y[10]*y[10] + 3./8.*y[1]*y[1] + 1./8.*y[2]*y[2];
    double gammaS2= 1./4.*y[2]*y[2] + 1./4.*y[4]*y[4];
    
    //--------------------------- TWO LOOP --------------------------//
    //g_1,g_2,g_3 (Two loops)
    dydt[7]=norm*y[7]*y[7]*y[7]*(bi[0] + norm*(bji[0][0]*y[7]*y[7] + bji[1][0]*y[8]*y[8] + bji[2][0]*y[9]*y[9] + ci[0]*y[10]*y[10]));//g_1
    dydt[8]=norm*y[8]*y[8]*y[8]*(bi[1] + norm*(bji[0][1]*y[7]*y[7] + bji[1][1]*y[8]*y[8] + bji[2][1]*y[9]*y[9] + ci[1]*y[10]*y[10]));//g_2
    dydt[9]=norm*y[9]*y[9]*y[9]*(bi[2] + norm*(bji[0][2]*y[7]*y[7] + bji[1][2]*y[8]*y[8] + bji[2][2]*y[9]*y[9] + ci[2]*y[10]*y[10]));//g_3
    //yt (Two loops)
    dydt[10]=norm*y[10]*(9./2.*y[10]*y[10] - 17./20.*y[7]*y[7] - 9./4.*y[8]*y[8] - 8.*y[9]*y[9] + norm*(3./8.*y[1]*y[1] + 1./8*y[2]*y[2] - 3.*y[10]*y[10]*y[1] - 12.*y[10]*y[10]*y[10]*y[10] + 393./80.*y[7]*y[7]*y[10]*y[10] + 255./16.*y[8]*y[8]*y[10]*y[10] + 36.*y[9]*y[9]*y[10]*y[10] + 1187./600.*y[7]*y[7]*y[7]*y[7] - 23./4.*y[8]*y[8]*y[8]*y[8] - 108.*y[9]*y[9]*y[9]*y[9] - 9./20.*y[7]*y[7]*y[8]*y[8] + 19./15.*y[7]*y[7]*y[9]*y[9] + 9.*y[8]*y[8]*y[9]*y[9]));
    //beta functions (Two Loops)
    //m2
    dydt[0]=norm*(3.*y[0]*y[1] + y[2]*y[3] + 2.*y[0]*gammaH1 + norm*(y[0]*(189./200.*y[7]*y[7]*y[7]*y[7] + 9./20.*y[7]*y[7]*y[8]*y[8] + 63./8.*y[8]*y[8]*y[8]*y[8] + 63./10.*y[7]*y[7]*y[1] + 63./2.*y[8]*y[8]*y[1] - 9./2.*y[1]*y[1] - 36.*y[10]*y[10]*y[1] - 1./2.*y[2]*y[2] + 2.*gammaH2 + 6.*y[1]*gammaH1) - y[2]*y[3]));
    //lambda
    dydt[1]=norm*(27./50.*y[7]*y[7]*y[7]*y[7] + 9./5.*y[7]*y[7]*y[8]*y[8] + 9./2.*y[8]*y[8]*y[8]*y[8] + 6.*y[1]*y[1] - 24.*y[10]*y[10]*y[10]*y[10] + y[2]*y[2] + 4.*y[1]*gammaH1 + norm*(-3411./500.*y[7]*y[7]*y[7]*y[7]*y[7]*y[7] - 1677./100.*y[7]*y[7]*y[7]*y[7]*y[8]*y[8] - 289./20.*y[7]*y[7]*y[8]*y[8]*y[8]*y[8] + 305./4.*y[8]*y[8]*y[8]*y[8]*y[8]*y[8] + 297./100.*y[7]*y[7]*y[7]*y[7]*y[1] + 9./2.*y[7]*y[7]*y[8]*y[8]*y[1] + 99./4.*y[8]*y[8]*y[8]*y[8]*y[1] + 54./5.*y[7]*y[7]*y[1]*y[1] + 54.*y[8]*y[8]*y[1]*y[1] - 21.*y[1]*y[1]*y[1] + y[10]*y[10]*(-171./25.*y[7]*y[7]*y[7]*y[7] + 126./5.*y[7]*y[7]*y[8]*y[8] - 9.*y[8]*y[8]*y[8]*y[8] - 72.*y[1]*y[1] + y[10]*y[10]*(-32./5.*y[7]*y[7] - 128.*y[9]*y[9] + 24.*y[1] + 120.*y[10]*y[10])) - 2.*y[2]*y[2]*y[2] - 7./2.*y[2]*y[2]*y[1] + 4.*y[1]*gammaH2 + 12.*y[1]*y[1]*gammaH1));
    //delta2
    dydt[2]=norm*(2.*y[2]*y[2] + 2.*y[2]*y[4] + 3.*y[2]*y[1] + 2*y[2]*gammaH1 + norm*y[2]*(189./200.*y[7]*y[7]*y[7]*y[7] + 9./20.*y[7]*y[7]*y[8]*y[8] + 63./8.*y[8]*y[8]*y[8]*y[8] + y[2]*(6./5.*y[7]*y[7] + 6.*y[8]*y[8] - 12.*y[10]*y[10]) + y[1]*(63./10.*y[7]*y[7] + 63./2.*y[8]*y[8] - 36.*y[10]*y[10]) - 3.*y[4]*y[4] - 6.*y[4]*y[2] - 7./2.*y[2]*y[2] - 9.*y[2]*y[1] - 9./2.*y[1]*y[1] + 2.*(gammaH2 + gammaS2 + (y[2] + 3.*y[1])*gammaH1)));
    //b2
    dydt[3]=norm*(2*y[3]*y[4] + 2*y[0]*y[2] + norm*(y[0]*y[2]*(21./5.*y[7]*y[7] + 21.*y[8]*y[8] - 24.*y[10]*y[10]) - 3.*y[3]*y[4]*y[4] - y[3]*y[2]*y[2] - 2.*y[0]*y[2]*y[2] + 4.*y[0]*y[2]*gammaH1 + 2.*y[3]*gammaS2));
    //d2
    dydt[4]=norm*(5*y[4]*y[4] + 2*y[2]*y[2] + norm*(-16.*y[4]*y[4]*y[4] - 6.*y[4]*y[2]*y[2] - 4.*y[2]*y[2]*y[2] + y[2]*y[2]*(21./5.*y[7]*y[7] + 21.*y[8]*y[8] - 24.*y[10]*y[10]) + 4*y[2]*y[2]*gammaH1 + 4.*y[4]*gammaS2));
    //b1
    dydt[5]=norm*y[5]*(y[4] - norm*(2.*y[4]*y[4] + y[2]*y[2] - 2.*gammaS2));
    //a1
    dydt[6]=norm*norm*y[6]*gammaS2;
    
}

#ifdef HBMODE
#ifdef HDECAYMODE
bool xSM2::PrepareHiggsBoundsInput(){
	
  /////////////////////////////////////////////////////////
  //Passing couplings, masses and decay properties to HB //
  /////////////////////////////////////////////////////////

  //Higgs (H0)
  //for mix_all nHzero=3 and the cycle goes threw all non-zero masses
  //with a DM candidate, nHzero is 2 and the cycle only goes from M1 to M2 (M2 is the higgs mass)

  compute_alphas();
    
  if(DM_candidate){
    CreateInputFileHdecayCsinglet(Cp.Mass[h[0]],Cp.Mass[h[1]],Cp.Mass[h[2]],alpha[1],alpha[2],alpha[3],Phi[4],Cp.L[6],4);
  }
  else{
    CreateInputFileHdecayCsinglet(Cp.Mass[h[0]],Cp.Mass[h[1]],Cp.Mass[h[2]],alpha[1],alpha[2],alpha[3],Phi[4],Cp.L[6],3);
  }
  HdecayCalcSing(HdecayHsing);
  if(HdecayHsing["H1Width"]==0){
    std::clog<<"All SM Hdecay partial Widths are 0 (this is likely to happen for small masses, current mass for lightest scalar state is "<<Cp.Mass[h[0]]<<"). This point will be rejected."<<endl;
      return false;
    }

    //Branching ratios and total width
  HiggsBoundsAnalysis.BR_hjss[0] = HdecayHsing["BR(H1 -> s sbar)"];
  HiggsBoundsAnalysis.BR_hjcc[0] = HdecayHsing["BR(H1 -> c cbar)"];
  HiggsBoundsAnalysis.BR_hjbb[0] = HdecayHsing["BR(H1 -> b bbar)"];
  HiggsBoundsAnalysis.BR_hjtt[0] = HdecayHsing["BR(H1 -> t tbar)"];
  HiggsBoundsAnalysis.BR_hjmumu[0] = HdecayHsing["BR(H1 -> mu+ mu-)"];
  HiggsBoundsAnalysis.BR_hjtautau[0] = HdecayHsing["BR(H1 -> tau+ tau-)"];
  HiggsBoundsAnalysis.BR_hjWW[0] = HdecayHsing["BR(H1 -> W+ W-)"];
  HiggsBoundsAnalysis.BR_hjZZ[0] = HdecayHsing["BR(H1 -> Z Z)"];
  HiggsBoundsAnalysis.BR_hjZga[0]= HdecayHsing["BR(H1 -> Z gamma)"];
  HiggsBoundsAnalysis.BR_hjgaga[0] = HdecayHsing["BR(H1 -> gamma gamma)"];
  HiggsBoundsAnalysis.BR_hjgg[0] = HdecayHsing["BR(H1 -> g g)"];
  HiggsBoundsAnalysis.MhGammaTot[0] = HdecayHsing["H1Width"];
  
  //Branching ratios and total width
  HiggsBoundsAnalysis.BR_hjss[1] = HdecayHsing["BR(H2 -> s sbar)"];
  HiggsBoundsAnalysis.BR_hjcc[1] = HdecayHsing["BR(H2 -> c cbar)"];
  HiggsBoundsAnalysis.BR_hjbb[1] = HdecayHsing["BR(H2 -> b bbar)"];
  HiggsBoundsAnalysis.BR_hjtt[1] = HdecayHsing["BR(H2 -> t tbar)"];
  HiggsBoundsAnalysis.BR_hjmumu[1] = HdecayHsing["BR(H2 -> mu+ mu-)"];
  HiggsBoundsAnalysis.BR_hjtautau[1] = HdecayHsing["BR(H2 -> tau+ tau-)"];
  HiggsBoundsAnalysis.BR_hjWW[1] = HdecayHsing["BR(H2 -> W+ W-)"];
  HiggsBoundsAnalysis.BR_hjZZ[1] = HdecayHsing["BR(H2 -> Z Z)"];
  HiggsBoundsAnalysis.BR_hjZga[1]= HdecayHsing["BR(H2 -> Z gamma)"];
  HiggsBoundsAnalysis.BR_hjgaga[1] = HdecayHsing["BR(H2 -> gamma gamma)"];
  HiggsBoundsAnalysis.BR_hjgg[1] = HdecayHsing["BR(H2 -> g g)"];
  HiggsBoundsAnalysis.MhGammaTot[1] = HdecayHsing["H2Width"];
  
  //Branching ratios and total width
  HiggsBoundsAnalysis.BR_hjss[2] = HdecayHsing["BR(H3 -> s sbar)"];
  HiggsBoundsAnalysis.BR_hjcc[2] = HdecayHsing["BR(H3 -> c cbar)"];
  HiggsBoundsAnalysis.BR_hjbb[2] = HdecayHsing["BR(H3 -> b bbar)"];
  HiggsBoundsAnalysis.BR_hjtt[2] = HdecayHsing["BR(H3 -> t tbar)"];
  HiggsBoundsAnalysis.BR_hjmumu[2] = HdecayHsing["BR(H3 -> mu+ mu-)"];
  HiggsBoundsAnalysis.BR_hjtautau[2] = HdecayHsing["BR(H3 -> tau+ tau-)"];
  HiggsBoundsAnalysis.BR_hjWW[2] = HdecayHsing["BR(H3 -> W+ W-)"];
  HiggsBoundsAnalysis.BR_hjZZ[2] = HdecayHsing["BR(H3 -> Z Z)"];
  HiggsBoundsAnalysis.BR_hjZga[2]= HdecayHsing["BR(H3 -> Z gamma)"];
  HiggsBoundsAnalysis.BR_hjgaga[2] = HdecayHsing["BR(H3 -> gamma gamma)"];
  HiggsBoundsAnalysis.BR_hjgg[2] = HdecayHsing["BR(H3 -> g g)"];
  HiggsBoundsAnalysis.MhGammaTot[2] = HdecayHsing["H3Width"];
  
  for(int i=0;i!=nHzero;++i){
    
    double Mi2sq=Mmixing[h[i]][2]*Mmixing[h[i]][2];
    
    HiggsBoundsAnalysis.Mh[i]=Cp.Mass[h[i]];
    HiggsBoundsAnalysis.CP[i]=1;

    //Neutral Higgs hadronic input
    //LEP
    HiggsBoundsAnalysis.CS_lep_hjZ_ratio[i]=Mi2sq;
    HiggsBoundsAnalysis.CS_lep_bbhj_ratio[i]=Mi2sq; 
    HiggsBoundsAnalysis.CS_lep_tautauhj_ratio[i]=Mi2sq; 
    //Tevatron
    HiggsBoundsAnalysis.CS_tev_hj_ratio[i]=Mi2sq;
    HiggsBoundsAnalysis.CS_tev_hjb_ratio[i]=Mi2sq;
    HiggsBoundsAnalysis.CS_tev_hjW_ratio[i]=Mi2sq;
    HiggsBoundsAnalysis.CS_tev_hjZ_ratio[i]=Mi2sq;
    HiggsBoundsAnalysis.CS_tev_vbf_ratio[i]=Mi2sq; 
    HiggsBoundsAnalysis.CS_tev_tthj_ratio[i]=Mi2sq;
    //LHC7
    HiggsBoundsAnalysis.CS_lhc7_hj_ratio[i]=Mi2sq;
    HiggsBoundsAnalysis.CS_lhc7_hjb_ratio[i]=Mi2sq;
    HiggsBoundsAnalysis.CS_lhc7_hjW_ratio[i]=Mi2sq;
    HiggsBoundsAnalysis.CS_lhc7_hjZ_ratio[i]=Mi2sq;
    HiggsBoundsAnalysis.CS_lhc7_vbf_ratio[i]=Mi2sq;
    HiggsBoundsAnalysis.CS_lhc7_tthj_ratio[i]=Mi2sq;
    //LHC8
    HiggsBoundsAnalysis.CS_lhc8_hj_ratio[i]=Mi2sq;
    HiggsBoundsAnalysis.CS_lhc8_hjb_ratio[i]=Mi2sq;
    HiggsBoundsAnalysis.CS_lhc8_hjW_ratio[i]=Mi2sq;
    HiggsBoundsAnalysis.CS_lhc8_hjZ_ratio[i]=Mi2sq;
    HiggsBoundsAnalysis.CS_lhc8_vbf_ratio[i]=Mi2sq;
    HiggsBoundsAnalysis.CS_lhc8_tthj_ratio[i]=Mi2sq;
  }

  //branching ratios for higgs to 2 higgs decay
  if(DM_candidate){
    HiggsBoundsAnalysis.BR_hjhihi[0][1] = HdecayHsing["BR(H1 -> H2 H2)"];
    HiggsBoundsAnalysis.BR_hjhihi[1][0] = HdecayHsing["BR(H2 -> H1 H1)"];
    HiggsBoundsAnalysis.BR_hjinvisible[0] = HdecayHsing["BR(H1 -> H3 H3)"];
    HiggsBoundsAnalysis.BR_hjinvisible[1] = HdecayHsing["BR(H2 -> H3 H3)"];
  }
  else{
    HiggsBoundsAnalysis.BR_hjhihi[0][1] = HdecayHsing["BR(H1 -> H2 H2)"];
    HiggsBoundsAnalysis.BR_hjhihi[0][2] = HdecayHsing["BR(H1 -> H3 H3)"];
    HiggsBoundsAnalysis.BR_hjhihi[1][0] = HdecayHsing["BR(H2 -> H1 H1)"];
    HiggsBoundsAnalysis.BR_hjhihi[1][2] = HdecayHsing["BR(H2 -> H3 H3)"];
    HiggsBoundsAnalysis.BR_hjhihi[2][0] = HdecayHsing["BR(H3 -> H1 H1)"];
    HiggsBoundsAnalysis.BR_hjhihi[2][1] = HdecayHsing["BR(H3 -> H2 H2)"];
  }
  
  return true;
  //ADD THESE TO HDECAYWRAPPER FOR SM FUNCTION
  //HiggsBoundsAnalysis.BR_tWpb[0] = HdecayTop.at("BR(t -> b W+)");
  //HiggsBoundsAnalysis.BR_tHpjb[0] = HdecayTop.at("BR(t -> b H+)");

}
#endif
#endif

double xSM2::compute_scalar_widths(void){
//Computes the scallar widths for mix_all 
  if(DM_candidate){
    double cosphi=Mmixing[0][2];
    double sinphi=-Mmixing[1][2];
    
    //DM cadidate does not decay into other scalars
    g012=0;
    g200=0;
    g211=0;
    
    g022= (Cp.L[2]*Phi[2]*cosphi + Cp.L[4]*Phi[4]*sinphi)/2.0;
    
    g122= (Cp.L[4]*Phi[4]*cosphi - Cp.L[2]*Phi[2]*sinphi)/2.0;
    
    g011= 1.5*Cp.L[1]*Phi[2]*cosphi*pow(sinphi,2) + 0.5*Cp.L[2]*Phi[2]*pow(cosphi,3) - Cp.L[2]*Phi[4]*sinphi*pow(cosphi,2) - Cp.L[2]*Phi[2]*cosphi*pow(sinphi,2) + 0.5*Cp.L[2]*Phi[4]*pow(sinphi,3) + 1.5*Cp.L[4]*Phi[4]*sinphi*pow(cosphi,2);
    
    g100= -1.5*Cp.L[1]*Phi[2]*sinphi*pow(cosphi,2) + 0.5*Cp.L[2]*Phi[4]*pow(cosphi,3) + Cp.L[2]*Phi[2]*sinphi*pow(cosphi,2) - Cp.L[2]*Phi[4]*cosphi*pow(sinphi,2) - 0.5*Cp.L[2]*Phi[2]*pow(sinphi,3) + 1.5*Cp.L[4]*Phi[4]*cosphi*pow(sinphi,2);
    
  }
  else{
    g011 = 3.0/4.0*Cp.L[1]* Phi[2]* Mmixing[0][2]* pow(Mmixing[1][2],2) + 1.0/2.0* Cp.L[2]* Phi[2]* Mmixing[0][4]* Mmixing[1][4]* Mmixing[1][2] +
    1.0/2.0* Cp.L[2]* Phi[2]* Mmixing[0][5]* Mmixing[1][5]* Mmixing[1][2] + 1.0/4.0* Cp.L[2]* Phi[2]* Mmixing[0][2]*pow( Mmixing[1][4],2) + 
    1.0/4.0* Cp.L[2]* Phi[2]* Mmixing[0][2]* pow(Mmixing[1][5],2) + 1.0/4.0* Cp.L[2]* Phi[5]* Mmixing[0][5]* pow(Mmixing[1][2],2) + 
    1.0/2.0* Cp.L[2]* Phi[5] *Mmixing[0][2]* Mmixing[1][5]* Mmixing[1][2] + 1.0/4.0* Cp.L[4]* Phi[5]* Mmixing[0][5]* pow(Mmixing[1][4],2) +
    3.0/4.0* Cp.L[4]* Phi[5]* Mmixing[0][5]* pow(Mmixing[1][5],2) + 1.0/2.0* Cp.L[4] *Phi[5]* Mmixing[0][4]* Mmixing[1][4]* Mmixing[1][5] + 
    1.0/4.0* Cp.L[2]* Phi[4]* Mmixing[0][4]* pow(Mmixing[1][2],2) + 1.0/2.0* Cp.L[2]* Phi[4] *Mmixing[0][2]* Mmixing[1][4] *Mmixing[1][2] + 
    3.0/4.0* Cp.L[4]* Phi[4]* Mmixing[0][4]* pow(Mmixing[1][4],2) + 1.0/4.0* Cp.L[4]* Phi[4]* Mmixing[0][4]* pow(Mmixing[1][5],2) +
    1.0/2.0* Cp.L[4]* Phi[4] *Mmixing[0][5]* Mmixing[1][4] *Mmixing[1][5];

    g022 = 3.0/4.0* Cp.L[1]* Phi[2]*Mmixing[0][2]* pow(Mmixing[2][2],2) + 1.0/2.0* Cp.L[2]* Phi[2]* Mmixing[0][4]* Mmixing[2][4]* Mmixing[2][2] +
    1.0/2.0* Cp.L[2]* Phi[2]* Mmixing[0][5]* Mmixing[2][5]* Mmixing[2][2] + 1.0/4.0* Cp.L[2]* Phi[2]* Mmixing[0][2]* pow(Mmixing[2][4],2) + 
    1.0/4.0* Cp.L[2]* Phi[2]* Mmixing[0][2]* pow(Mmixing[2][5],2) + 1.0/4.0* Cp.L[2]* Phi[5]* Mmixing[0][5]* pow(Mmixing[2][2],2) + 
    1.0/2.0* Cp.L[2]* Phi[5] *Mmixing[0][2]* Mmixing[2][5]* Mmixing[2][2] +  1.0/4.0* Cp.L[4]* Phi[5]* Mmixing[0][5]* pow(Mmixing[2][4],2) + 
    3.0/4.0* Cp.L[4]* Phi[5]* Mmixing[0][5]* pow(Mmixing[2][5],2) + 1.0/2.0* Cp.L[4]* Phi[5]* Mmixing[0][4]* Mmixing[2][4]* Mmixing[2][5] + 
    1.0/4.0* Cp.L[2]* Phi[4]* Mmixing[0][4]* pow(Mmixing[2][2],2) + 1.0/2.0* Cp.L[2]* Phi[4]* Mmixing[0][2]* Mmixing[2][4]* Mmixing[2][2] + 
    3.0/4.0* Cp.L[4]* Phi[4]* Mmixing[0][4]* pow(Mmixing[2][4],2) + 1.0/4.0* Cp.L[4]* Phi[4]* Mmixing[0][4]* pow(Mmixing[2][5],2) + 
    1.0/2.0* Cp.L[4]* Phi[4]* Mmixing[0][5]* Mmixing[2][4]* Mmixing[2][5];

    g012 = 3.0/2.0* Cp.L[1]* Phi[2]* Mmixing[0][2]* Mmixing[1][2]* Mmixing[2][2] + 
    1.0/2.0* Cp.L[2]* Phi[2]* Mmixing[0][4]* Mmixing[1][4]* Mmixing[2][2] + 1.0/2.0* Cp.L[2]* Phi[2]* Mmixing[0][5]* Mmixing[1][5]* Mmixing[2][2] + 
    1.0/2.0* Cp.L[2]* Phi[2]* Mmixing[0][4]* Mmixing[1][2]* Mmixing[2][4] + 1.0/2.0* Cp.L[2]* Phi[2]* Mmixing[0][2]* Mmixing[1][4]* Mmixing[2][4] + 
    1.0/2.0* Cp.L[2]* Phi[2]* Mmixing[0][5]* Mmixing[1][2]* Mmixing[2][5] + 1.0/2.0* Cp.L[2]* Phi[2]* Mmixing[0][2]* Mmixing[1][5]* Mmixing[2][5] + 
    1.0/2.0* Cp.L[2]* Phi[5] *Mmixing[0][5]* Mmixing[1][2]* Mmixing[2][2] + 1.0/2.0* Cp.L[2]* Phi[5]* Mmixing[0][2]* Mmixing[1][5]* Mmixing[2][2] + 
    1.0/2.0* Cp.L[4]* Phi[5]* Mmixing[0][5]* Mmixing[1][4]* Mmixing[2][4] + 1.0/2.0* Cp.L[4]* Phi[5]* Mmixing[0][4]* Mmixing[1][5]* Mmixing[2][4] + 
    1.0/2.0* Cp.L[2]* Phi[5] *Mmixing[0][2] *Mmixing[1][2]* Mmixing[2][5] + 1.0/2.0* Cp.L[4]* Phi[5]* Mmixing[0][4]* Mmixing[1][4]* Mmixing[2][5] + 
    3.0/2.0* Cp.L[4]* Phi[5]* Mmixing[0][5]* Mmixing[1][5]* Mmixing[2][5] + 1.0/2.0* Cp.L[2]* Phi[4]* Mmixing[0][4]* Mmixing[1][2]* Mmixing[2][2] + 
    1.0/2.0* Cp.L[2]* Phi[4]* Mmixing[0][2]* Mmixing[1][4]* Mmixing[2][2] + 1.0/2.0* Cp.L[2]* Phi[4]* Mmixing[0][2]* Mmixing[1][2]* Mmixing[2][4] + 
    3.0/2.0* Cp.L[4]* Phi[4]* Mmixing[0][4]* Mmixing[1][4]* Mmixing[2][4] + 1.0/2.0* Cp.L[4]* Phi[4]* Mmixing[0][5]* Mmixing[1][5]* Mmixing[2][4] + 
    1.0/2.0* Cp.L[4]* Phi[4] *Mmixing[0][5]* Mmixing[1][4]* Mmixing[2][5] + 1.0/2.0* Cp.L[4]* Phi[4]* Mmixing[0][4]* Mmixing[1][5]* Mmixing[2][5];

    g100 = 3.0/4.0*Cp.L[1]*Phi[2]*Mmixing[1][2]*pow( Mmixing[0][2],2) + 1.0/2.0* Cp.L[2]* Phi[2]* Mmixing[0][4]* Mmixing[1][4]* Mmixing[0][2] + 
    1.0/2.0* Cp.L[2]* Phi[2]* Mmixing[0][5]* Mmixing[1][5]* Mmixing[0][2] + 1.0/4.0* Cp.L[2]* Phi[2] *pow(Mmixing[0][4],2)* Mmixing[1][2] + 
    1.0/4.0* Cp.L[2]* Phi[2]* pow(Mmixing[0][5],2)* Mmixing[1][2] + 1.0/4.0* Cp.L[2]* Phi[5]* Mmixing[1][5]* pow(Mmixing[0][2],2) + 
    1.0/2.0* Cp.L[2]* Phi[5] *Mmixing[0][5]* Mmixing[1][2]* Mmixing[0][2] + 1.0/2.0* Cp.L[4]* Phi[5]* Mmixing[0][4]* Mmixing[0][5]* Mmixing[1][4] +
    1.0/4.0* Cp.L[4]* Phi[5]* pow(Mmixing[0][4],2)* Mmixing[1][5] + 3.0/4.0* Cp.L[4]* Phi[5]* pow(Mmixing[0][5],2)* Mmixing[1][5] + 
    1.0/4.0* Cp.L[2]* Phi[4]* Mmixing[1][4]* pow(Mmixing[0][2],2) + 1.0/2.0* Cp.L[2]* Phi[4] *Mmixing[0][4]* Mmixing[1][2]* Mmixing[0][2] + 
    3.0/4.0* Cp.L[4]* Phi[4]* pow(Mmixing[0][4],2)* Mmixing[1][4] + 1.0/4.0* Cp.L[4]* Phi[4]* pow(Mmixing[0][5],2) *Mmixing[1][4] + 
    1.0/2.0* Cp.L[4]* Phi[4] *Mmixing[0][4] *Mmixing[0][5]* Mmixing[1][5];

    g122 = 3.0/4.0* Cp.L[1]* Phi[2]* Mmixing[1][2]*pow(Mmixing[2][2],2)+1.0/2.0* Cp.L[2]* Phi[2] *Mmixing[1][4]* Mmixing[2][4]*Mmixing[2][2] +
    1.0/2.0* Cp.L[2]* Phi[2]* Mmixing[1][5]* Mmixing[2][5]* Mmixing[2][2] + 1.0/4.0* Cp.L[2]* Phi[2] *Mmixing[1][2]* pow(Mmixing[2][4],2) + 
    1.0/4.0* Cp.L[2]* Phi[2]* Mmixing[1][2]* pow(Mmixing[2][5],2) + 1.0/4.0* Cp.L[2]* Phi[5]* Mmixing[1][5]*pow( Mmixing[2][2],2) + 
    1.0/2.0* Cp.L[2]* Phi[5]* Mmixing[1][2]* Mmixing[2][5]* Mmixing[2][2] + 1.0/4.0* Cp.L[4]* Phi[5]* Mmixing[1][5]* pow(Mmixing[2][4],2) + 
    3.0/4.0* Cp.L[4]* Phi[5] *Mmixing[1][5]* pow(Mmixing[2][5],2) + 1.0/2.0* Cp.L[4]* Phi[5] *Mmixing[1][4]* Mmixing[2][4]* Mmixing[2][5] + 
    1.0/4.0* Cp.L[2]* Phi[4] *Mmixing[1][4]*pow( Mmixing[2][2],2) + 1.0/2.0* Cp.L[2]* Phi[4]* Mmixing[1][2]* Mmixing[2][4]* Mmixing[2][2] + 
    3.0/4.0* Cp.L[4]* Phi[4]* Mmixing[1][4]*pow( Mmixing[2][4],2) + 1.0/4.0* Cp.L[4]* Phi[4]* Mmixing[1][4]* pow(Mmixing[2][5],2) +
    1.0/2.0* Cp.L[4]* Phi[4]* Mmixing[1][5]* Mmixing[2][4]* Mmixing[2][5];

    g200 = 3.0/4.0* Cp.L[1]*Phi[2]* Mmixing[2][2]*pow(Mmixing[0][2],2) + 1.0/2.0* Cp.L[2]* Phi[2] *Mmixing[0][4]* Mmixing[2][4]*Mmixing[0][2] + 
    1.0/2.0* Cp.L[2]* Phi[2] *Mmixing[0][5]* Mmixing[2][5]* Mmixing[0][2] + 1.0/4.0* Cp.L[2]* Phi[2] *pow(Mmixing[0][4],2)* Mmixing[2][2] + 
    1.0/4.0* Cp.L[2]* Phi[2] *pow(Mmixing[0][5],2)* Mmixing[2][2] + 1.0/4.0* Cp.L[2]* Phi[5]* Mmixing[2][5]* pow(Mmixing[0][2],2) + 
    1.0/2.0* Cp.L[2]* Phi[5] *Mmixing[0][5]* Mmixing[2][2] *Mmixing[0][2] + 1.0/2.0* Cp.L[4] *Phi[5] *Mmixing[0][4]* Mmixing[0][5]* Mmixing[2][4] +
    1.0/4.0* Cp.L[4]* Phi[5] *pow(Mmixing[0][4],2)* Mmixing[2][5] + 3.0/4.0* Cp.L[4]* Phi[5]* pow(Mmixing[0][5],2)* Mmixing[2][5] + 
    1.0/4.0* Cp.L[2]* Phi[4]* Mmixing[2][4]* pow(Mmixing[0][2],2) + 1.0/2.0* Cp.L[2]* Phi[4]* Mmixing[0][4]* Mmixing[2][2]* Mmixing[0][2] + 
    3.0/4.0* Cp.L[4]* Phi[4]*pow( Mmixing[0][4],2)* Mmixing[2][4] + 1.0/4.0* Cp.L[4]* Phi[4]* pow(Mmixing[0][5],2)* Mmixing[2][4] + 
    1.0/2.0* Cp.L[4]* Phi[4] *Mmixing[0][4]* Mmixing[0][5] *Mmixing[2][5];

    g211 =3.0/4.0* Cp.L[1]* Phi[2]* Mmixing[2][2]*pow( Mmixing[1][2],2) + 1.0/2.0* Cp.L[2]* Phi[2]* Mmixing[1][4]*Mmixing[2][4]* Mmixing[1][2] +
    1.0/2.0* Cp.L[2]* Phi[2]* Mmixing[1][5]* Mmixing[2][5]* Mmixing[1][2] + 1.0/4.0* Cp.L[2]* Phi[2]* pow(Mmixing[1][4],2)* Mmixing[2][2] + 
    1.0/4.0* Cp.L[2]* Phi[2] *pow(Mmixing[1][5],2)* Mmixing[2][2] + 1.0/4.0* Cp.L[2]* Phi[5] *Mmixing[2][5]* pow(Mmixing[1][2],2) + 
    1.0/2.0* Cp.L[2]* Phi[5] *Mmixing[1][5]* Mmixing[2][2]* Mmixing[1][2] + 1.0/2.0* Cp.L[4]* Phi[5]* Mmixing[1][4]* Mmixing[1][5]* Mmixing[2][4] + 
    1.0/4.0* Cp.L[4]* Phi[5] *pow(Mmixing[1][4],2)* Mmixing[2][5] + 3.0/4.0* Cp.L[4]* Phi[5]* pow(Mmixing[1][5],2) *Mmixing[2][5] + 
    1.0/4.0* Cp.L[2]* Phi[4]* Mmixing[2][4]* pow(Mmixing[1][2],2) + 1.0/2.0* Cp.L[2]* Phi[4] *Mmixing[1][4]* Mmixing[2][2]* Mmixing[1][2] + 
    3.0/4.0* Cp.L[4]* Phi[4] *pow(Mmixing[1][4],2) *Mmixing[2][4] + 1.0/4.0* Cp.L[4]* Phi[4]* pow(Mmixing[1][5],2)* Mmixing[2][4] + 
    1.0/2.0* Cp.L[4]* Phi[4] *Mmixing[1][4]* Mmixing[1][5]* Mmixing[2][5];
  
  //multiply couplings of decay to two identical particles by a factor of two comming from two possible contractions for identical fields in the final state, from Wick's theorem
    g011=2.0*g011;
    g022=2.0*g022;
    g100=2.0*g100;
    g122=2.0*g122;
    g200=2.0*g200;
    g211=2.0*g211;
  }
  
  //Compute Gamma functions
  double Gfrac1,Gfrac2;
  
  Gfrac1=Cp.Mass[1]*Cp.Mass[1]/Cp.Mass[0]/Cp.Mass[0];	//Gamma 011 & 100
  if(Gfrac1<.25){
    Gamma011=Gamma_const*g011*g011/Cp.Mass[0]*sqrt(1-4*Gfrac1);
    Gamma100=0;
  }
  else if(Gfrac1>4){
    Gamma100=Gamma_const*g100*g100/Cp.Mass[1]*sqrt(1-4./Gfrac1);
    Gamma011=0;
  }
  else{
    Gamma011=0;
    Gamma100=0;
  }
  Gfrac1=Cp.Mass[2]*Cp.Mass[2]/Cp.Mass[0]/Cp.Mass[0];	//Gamma 022 & 200
  if(Gfrac1<.25){
    Gamma022=Gamma_const*g022*g022/Cp.Mass[0]*sqrt(1-4*Gfrac1);
    Gamma200=0;
  }
  else if(Gfrac1>4){
    Gamma200=Gamma_const*g200*g200/Cp.Mass[2]*sqrt(1-4./Gfrac1);
    Gamma022=0;
  }
  else{
    Gamma022=0;
    Gamma200=0;
  }
  Gfrac1=Cp.Mass[2]*Cp.Mass[2]/Cp.Mass[1]/Cp.Mass[1];	//Gamma 122 & 211
  if(Gfrac1<.25){
    Gamma122=Gamma_const*g122*g122/Cp.Mass[1]*sqrt(1-4*Gfrac1);
    Gamma211=0;
  }
  else if(Gfrac1>4){
    Gamma211=Gamma_const*g211*g211/Cp.Mass[2]*sqrt(1-4./Gfrac1);
    Gamma122=0;
  }
  else{
    Gamma122=0;
    Gamma211=0;
  }
  
  Gfrac1=pow(Cp.Mass[1]+Cp.Mass[2],2)/Cp.Mass[0]/Cp.Mass[0];
  Gfrac2=pow(Cp.Mass[1]-Cp.Mass[2],2)/Cp.Mass[0]/Cp.Mass[0];
  if(Gfrac1<1 && Gfrac2<1)
    Gamma012=2*Gamma_const*g012*g012/Cp.Mass[0]*sqrt(1-Gfrac1)*sqrt(1-Gfrac2);
  else
    Gamma012=0;
  Gfrac1=pow(Cp.Mass[0]+Cp.Mass[2],2)/Cp.Mass[1]/Cp.Mass[1];
  Gfrac2=pow(Cp.Mass[0]-Cp.Mass[2],2)/Cp.Mass[1]/Cp.Mass[1];
  if(Gfrac1<1 && Gfrac2<1)
    Gamma102=2*Gamma_const*g012*g012/Cp.Mass[1]*sqrt(1-Gfrac1)*sqrt(1-Gfrac2);
  else
    Gamma102=0;
  Gfrac1=pow(Cp.Mass[0]+Cp.Mass[1],2)/Cp.Mass[2]/Cp.Mass[2];
  Gfrac2=pow(Cp.Mass[0]-Cp.Mass[1],2)/Cp.Mass[2]/Cp.Mass[2];
  if(Gfrac1<1 && Gfrac2<1)
    Gamma201=2*Gamma_const*g012*g012/Cp.Mass[2]*sqrt(1-Gfrac1)*sqrt(1-Gfrac2);
  else
    Gamma201=0;
  
  GammaS[0]=Gamma011+Gamma022+Gamma012;
  GammaS[1]=Gamma100+Gamma122+Gamma102;
  GammaS[2]=Gamma200+Gamma211+Gamma201;
}

void xSM2::compute_alphas(void){
  
  //Order the masses of the final states
  h[0]=0;	//h0 is a dummy variable
  h[1]=1;	//h0 is a dummy variable
  h[2]=2;	//h0 is a dummy variable
  size_t temp;
  
  if(DM_candidate){
    if(Cp.Mass[h[0]]>Cp.Mass[h[1]]){
      h[1]=0;
      h[0]=1;
    }
    int reflect1=1,reflect2=1;
    double det=Mmixing[h[0]][2]*Mmixing[h[1]][4] - Mmixing[h[0]][4]*Mmixing[h[1]][2];
    if(det>0){
      if(Mmixing[h[0]][2]<0){
	reflect1=-1;
	reflect2=-1;
      }
    }
    else{
      if(Mmixing[h[0]][2]>0)
	reflect2=-1;
      else
	reflect1=-1;
    }
    //clog<<"det: "<<det<<endl;
    
    //compute the alphas from the mixing matrix
    alpha[1]=asin(reflect1*Mmixing[h[0]][4]);
    
    /*double diff=abs(reflect1*Mmixing[h[0]][2]-cos(alpha[1]))+abs(reflect1*Mmixing[h[0]][4]-sin(alpha[1]))+abs(reflect2*Mmixing[h[1]][2]+sin(alpha[1]))+abs(reflect2*Mmixing[h[1]][4]-cos(alpha[1]));
      if(diff>1e-14){
      clog<<"Mixing Matrix after correcting for a possible reflection is:"<<endl;
      clog<<reflect1*Mmixing[h[0]][2]<<"\t"<<reflect1*Mmixing[h[0]][4]<<endl;
      clog<<reflect2*Mmixing[h[1]][2]<<"\t"<<reflect2*Mmixing[h[1]][4]<<endl;
    clog<<"and the one derived from alpha is:"<<endl;
    clog<<cos(alpha[1])<<"\t"<<sin(alpha[1])<<endl;
      clog<<-sin(alpha[1])<<"\t"<<cos(alpha[1])<<endl<<endl; 
    
      clog<<"The matrix of differences is:"<<endl;
      clog<<abs(reflect1*Mmixing[h[0]][2]-cos(alpha[1]))<<"\t"<<abs(reflect1*Mmixing[h[0]][4]-sin(alpha[1]))<<endl;
      clog<<abs(reflect2*Mmixing[h[1]][2]+sin(alpha[1]))<<"\t"<<abs(reflect2*Mmixing[h[1]][4]-cos(alpha[1]))<<endl<<endl;   
      }*/
    alpha[2]=0.;
    alpha[3]=0.;
  }
  //in the broken phase
  else{
    if(Cp.Mass[h[0]]>Cp.Mass[h[2]]){
      temp=h[2];
      h[2]=h[0];
      h[0]=temp;
    }
    if(Cp.Mass[h[0]]>Cp.Mass[h[1]]){
      temp=h[1];
      h[1]=h[0];
      h[0]=temp;
    }
    else if(Cp.Mass[h[1]]>Cp.Mass[h[2]]){
      temp=h[2];
      h[2]=h[1];
      h[1]=temp;
    }
    //clog<<"Order: "<<h[0]<<"; "<<h[1]<<"; "<<h[2]<<endl;
    
    double det=-Mmixing[h[0]][5]*Mmixing[h[1]][4]*Mmixing[h[2]][2] + Mmixing[h[0]][4]*Mmixing[h[1]][5]*Mmixing[h[2]][2] + Mmixing[h[0]][5]*Mmixing[h[1]][2]*Mmixing[h[2]][4] - Mmixing[h[0]][2]*Mmixing[h[1]][5]* Mmixing[h[2]][4] - Mmixing[h[0]][4]*Mmixing[h[1]][2]*Mmixing[h[2]][5] + Mmixing[h[0]][2]*Mmixing[h[1]][4]*Mmixing[h[2]][5];
    
    //clog<<"det= "<<det<<endl;
    int reflect1=1,reflect2=1,reflect3=1;
    if(det>0){
      //clog<<"positive determinant"<<endl;
      if(Mmixing[h[0]][2]<0 && Mmixing[h[2]][5]<0){
	reflect1=-1;
	reflect3=-1;
      }
      else if(Mmixing[h[0]][2]<0){
	reflect1=-1;
	reflect2=-1;
      }
      else if(Mmixing[h[2]][5]<0){
	reflect3=-1;
	reflect2=-1;
      }
    }
    else{
      //clog<<"negative determinant"<<endl;
      if(Mmixing[h[0]][2]<0 && Mmixing[h[2]][5]<0){
	//clog<<"case1"<<endl;
	reflect1=-1;
	reflect2=-1;
	reflect3=-1;
      }
      else if(Mmixing[h[0]][2]<0){
	//clog<<"case2"<<endl;
	reflect1=-1;
      }
      else if(Mmixing[h[2]][5]<0){
	//clog<<"case3"<<endl;
	reflect3=-1;
      }
      else{//both positive
	//clog<<"case4"<<endl;
	reflect2=-1;
      }
    }
    
    
    //compute the alphas from the mixing matrix
    alpha[2]=asin(reflect1*Mmixing[h[0]][5]);
    alpha[1]=asin(reflect1*Mmixing[h[0]][4]/cos(alpha[2]));    
    alpha[3]=asin(reflect2*Mmixing[h[1]][5]/cos(alpha[2]));
    
    //CODE USED TO TEST THE REFLECTIONS
    /*double c1=cos(alpha[1]), s1=sin(alpha[1]), c2=cos(alpha[2]), s2=sin(alpha[2]),c3=cos(alpha[3]),s3=sin(alpha[3]),R1h=c1*c2,R1s=c2*s1,R1a=s2,R2h=-(c3*s1) - c1*s2*s3,R2s=c1*c3 - s1*s2*s3,R2a=c2*s3,R3h=-(c1*c3*s2) + s1*s3,R3s=-(c3*s1*s2) - c1*s3,R3a=c2*c3;
      if(abs(reflect1*Mmixing[h[0]][2]-R1h)>1e-8||abs(reflect1*Mmixing[h[0]][4]-R1s)>1e-8||abs(reflect1*Mmixing[h[0]][5]-R1a)>1e-8|| abs(reflect2*Mmixing[h[1]][2]-R2h)>1e-8||abs(reflect2*Mmixing[h[1]][4]-R2s)>1e-8||abs(reflect2*Mmixing[h[1]][5]-R2a)>1e-8|| abs(reflect3*Mmixing[h[2]][2]-R3h)>1e-8||abs(reflect3*Mmixing[h[2]][4]-R3s)>1e-8||abs(reflect3*Mmixing[h[2]][5]-R3a)>1e-8){
      clog<<"BAD ALPHAS!!!!!!!!!!!!!!!"<<endl;
      
      clog<<"The mixing matrix after suitable reflections is"<<endl;
      clog<<reflect1*Mmixing[h[0]][2]<<"\t";
      clog<<reflect1*Mmixing[h[0]][4]<<"\t";
      clog<<reflect1*Mmixing[h[0]][5]<<endl;
      
      
      clog<<reflect2*Mmixing[h[1]][2]<<"\t";
      clog<<reflect2*Mmixing[h[1]][4]<<"\t";
      clog<<reflect2*Mmixing[h[1]][5]<<endl;
      
      clog<<reflect3*Mmixing[h[2]][2]<<"\t";
      clog<<reflect3*Mmixing[h[2]][4]<<"\t";
      clog<<reflect3*Mmixing[h[2]][5]<<endl<<endl;
      clog<<" The matrix computed with  the alphas is"<<endl;

      clog<<R1h<<"\t";
      clog<<R1s<<"\t";
      clog<<R1a<<endl;
      
      clog<<R2h<<"\t";
      clog<<R2s<<"\t";
      clog<<R2a<<endl;
      
      clog<<R3h<<"\t";
      clog<<R3s<<"\t";
      clog<<R3a<<endl;
      
      clog<<"The differences are:"<<endl;
      clog<<reflect1*Mmixing[h[0]][2]-R1h<<"\t";
      clog<<reflect1*Mmixing[h[0]][4]-R1s<<"\t";
      clog<<reflect1*Mmixing[h[0]][5]-R1a<<endl;
      
      
      clog<<reflect2*Mmixing[h[1]][2]-R2h<<"\t";
      clog<<reflect2*Mmixing[h[1]][4]-R2s<<"\t";
      clog<<reflect2*Mmixing[h[1]][5]-R2a<<endl;
      
      clog<<reflect3*Mmixing[h[2]][2]-R3h<<"\t";
      clog<<reflect3*Mmixing[h[2]][4]-R3s<<"\t";
      clog<<reflect3*Mmixing[h[2]][5]-R3a<<endl<<endl;
    
      
      clog<<"ALPHAS/(Pi/2): "<<alpha[1]*2./acos(-1)<<"; "<<alpha[2]*2./acos(-1)<<"; "<<alpha[3]*2./acos(-1)<<endl;    
      
      
      }*/
    
  }
}

#ifdef HBMODE
void xSM2::OrderGammas(void){
  //makes GammaOrdered a vector with [h2->h1h1;h3->h1h1;h3->h1h2;h3->h2h2]
  //Called from compute_alphas()
  if(!DM_candidate){
    GammaOrdered[0]=HdecayHsing["BR(H2 -> H1 H1)"];
    GammaOrdered[1]=HdecayHsing["BR(H3 -> H1 H1)"];
    GammaOrdered[2]=HdecayHsing["BR(H3 -> H1 H2)"];
    GammaOrdered[3]=HdecayHsing["BR(H3 -> H2 H2)"];
  }
  else {
    GammaOrdered[0]=HdecayHsing["BR(H2 -> H1 H1)"];
    GammaOrdered[1]=HdecayHsing["BR(H1 -> H3 H3)"];
    GammaOrdered[2]=HdecayHsing["BR(H2 -> H3 H3)"];
    GammaOrdered[3]=0.;
  }
}
#endif

#ifdef MICROMEGASMODE
void xSM2::assignMOvalues(){
  
  ////// Dark matter
  ////// Relic density

  double cosphi=Mmixing[0][2];
  double sinphi=-Mmixing[1][2];

  ForceUG=1;  // to Force Unitary Gauge assign 1 
 
  assignVal("msqr",Cp.L[0]);
  assignVal("lmbd",Cp.L[1]);
  assignVal("delta2",Cp.L[2]);
  assignVal("b2",Cp.L[3]);
  assignVal("d2",Cp.L[4]);
  assignVal("b1",Cp.L[5]);
  assignVal("ct",sinphi);		//WARNING: temporary fix to lanhep model file wich had h and s swaped from our convention
  assignVal("st",cosphi);		//WARNING: temporary fix to lanhep model file wich had h and s swaped from our convention
  assignVal("sigma",Phi[4]);
  assignVal("Mh",Cp.Mass[1]);		//WARNING: temporary fix to lanhep model file wich had h and s swaped from our convention
  assignVal("muS",Cp.Mass[0]);		//WARNING: temporary fix to lanhep model file wich had h and s swaped from our convention
  assignVal("MAs",Cp.Mass[2]);  
  char cdmName[10];
  int err=sortOddParticles(cdmName);
  if(err) { cerr<<"Can't calculate "<<cdmName<< endl; exit(EXIT_FAILURE);}
}
#endif
