/*/ Numerically compare first functions for SM + real singlet

STUchecker STU_SM_plus_real_singlet;
size_t nHzero=2,nHplus=0,nDoublet=1;
STU_SM_plus_real_singlet.setup(nHzero,nHplus,nDoublet);

double mH1=134.21,mH2=232.244,ca1=0.2324;
double sa1=sqrt(1-ca1*ca1);


// feed values into the STU input
// matrices
STU_SM_plus_real_singlet.m_U.at(0).at(0) = 1;

STU_SM_plus_real_singlet.m_Vr.at(0).at(0) = 0;
STU_SM_plus_real_singlet.m_Vr.at(0).at(1) = ca1;
STU_SM_plus_real_singlet.m_Vr.at(0).at(2) = -sa1;

STU_SM_plus_real_singlet.m_Vi.at(0).at(0) = 1;
STU_SM_plus_real_singlet.m_Vi.at(0).at(1) = 0;
STU_SM_plus_real_singlet.m_Vi.at(0).at(2) = 0;
//masses
STU_SM_plus_real_singlet.mHzero.at(0)=mH1;
STU_SM_plus_real_singlet.mHzero.at(1)=mH2;

//evaluate
bool result = STU_SM_plus_real_singlet.evaluate();

//Same calculation but with my functions
vector<double> Mass(2,0);
vector< vector<double> > Mixing(2,Mass);
Mass[0]=mH1;
Mass[1]=mH2;
Mixing[0][0]=ca1;
Mixing[0][1]=sa1;
Mixing[1][0]=-sa1;
Mixing[1][1]=ca1;

double myS=DS_nsinglets2(Mass,Mixing,0)-DSsm(mHref);
double myT=DT_nsinglets2(Mass,Mixing,0)-DTsm(mHref);
double myU=DU_nsinglets2(Mass,Mixing,0)-DUsm(mHref);

clog<<"\t Jonas \t\t Me \t\t diff"<<endl;
clog<<"S\t"<< STU_SM_plus_real_singlet.S<<"\t"<<myS<<"\t"<<(myS-STU_SM_plus_real_singlet.S)/sqrt(myS*myS+STU_SM_plus_real_singlet.S*STU_SM_plus_real_singlet.S)<<endl;
clog<<"T\t"<< STU_SM_plus_real_singlet.T<<"\t"<<myT<<"\t"<<(myT-STU_SM_plus_real_singlet.T)/sqrt(myT*myT+STU_SM_plus_real_singlet.T*STU_SM_plus_real_singlet.T)<<endl;
clog<<"U\t"<< STU_SM_plus_real_singlet.U<<"\t"<<myU<<"\t"<<(myU-STU_SM_plus_real_singlet.U)/sqrt(myU*myU+STU_SM_plus_real_singlet.U*STU_SM_plus_real_singlet.U)<<endl;
//*/

// Numerically compare functions in THDM


STUchecker STU_THDM;
size_t nHzero=3,nHplus=1,nDoublet=2;
STU_THDM.setup(nHzero,nHplus,nDoublet);

double mH1=134.21,mH2=232.244, mA=320.2342,mHc=200.345,alpha=0.532,beta=0.9722;
double ca1=cos(alpha),sb=sin(beta);
double sa1=sin(alpha),cb=cos(beta);


// feed values into the STU input
// matrices

    STU_THDM.m_U.at(0).at(0) = cb;
    STU_THDM.m_U.at(0).at(1) = -sb;
    STU_THDM.m_U.at(1).at(0) = sb;
    STU_THDM.m_U.at(1).at(1) = cb;
    STU_THDM.m_Vr.at(0).at(0) = 0;
    STU_THDM.m_Vr.at(0).at(1) = 0;
    STU_THDM.m_Vr.at(0).at(2) = ca1;
    STU_THDM.m_Vr.at(0).at(3) = -sa1;
    STU_THDM.m_Vr.at(1).at(0) = 0;
    STU_THDM.m_Vr.at(1).at(1) = 0;
    STU_THDM.m_Vr.at(1).at(2) = sa1;
    STU_THDM.m_Vr.at(1).at(3) = ca1;
    STU_THDM.m_Vi.at(0).at(0) = cb;
    STU_THDM.m_Vi.at(0).at(1) = -sb;
    STU_THDM.m_Vi.at(0).at(2) = 0;
    STU_THDM.m_Vi.at(0).at(3) = 0;
    STU_THDM.m_Vi.at(1).at(0) = sb;
    STU_THDM.m_Vi.at(1).at(1) = cb;
    STU_THDM.m_Vi.at(1).at(2) = 0;
    STU_THDM.m_Vi.at(1).at(3) = 0;
    //masses
    STU_THDM.mHzero.at(0)=mA;
    STU_THDM.mHzero.at(1)=mH1;
    STU_THDM.mHzero.at(2)=mH2;
    STU_THDM.mHplus.at(0)=mHc;

//evaluate
bool result = STU_THDM.evaluate();
// NOTE THAT JONAS AN I HAVE AN OPPOSITE CONVENTION ORGANISING H2 and H1!!!!
double myS=DS_2HDM(mH2,mH1,mA,mHc,alpha,beta)-DSsm(mHref);
double myT=DT_2HDM(mH2,mH1,mA,mHc,alpha,beta)-DTsm(mHref);
double myU=DU_2HDM(mH2,mH1,mA,mHc,alpha,beta)-DUsm(mHref);

clog<<"\t Jonas \t\t Me \t\t diff"<<endl;
clog<<"S\t"<< STU_THDM.S<<"\t"<<myS<<"\t"<<2*(myS-STU_THDM.S)/sqrt(myS*myS+STU_THDM.S*STU_THDM.S)<<endl;
clog<<"T\t"<< STU_THDM.T<<"\t"<<myT<<"\t"<<2*(myT-STU_THDM.T)/sqrt(myT*myT+STU_THDM.T*STU_THDM.T)<<endl;
clog<<"U\t"<< STU_THDM.U<<"\t"<<myU<<"\t"<<2*(myU-STU_THDM.U)/sqrt(myU*myU+STU_THDM.U*STU_THDM.U)<<endl;

// Numerically test all functions. 
